"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-

from flask_login import UserMixin
from itsdangerous import URLSafeTimedSerializer
from constants import SECRET_KEY
from utilities.user_manager import UserManager
from libs import base
from libs import citrusleaf as cl

login_serializer = URLSafeTimedSerializer(SECRET_KEY)


class User(UserMixin):
    """
    User Class for flask-Login
    """
    def __init__(self, userid, password, host=None, roles=[], pl_password=None):
        self.id = userid
        self.password = password
        self.host = host
        self.roles = roles
        self.plain_password = pl_password
        if host:
            try:
                self.ip, self.port = host.split(":")
            except:
                self.ip, self.port = None, None

    def get_auth_token(self):
        """
        Encode a secure token for cookie
        """
        data = [str(self.id), self.password]
        return login_serializer.dumps(data)


    def is_authenticated(self):
        return True

    @staticmethod
    def get(userid, password, host=None, roles=[], pl_password=None):
        """
        Static method to search the database and see if userid exists.  If it
        does exist then return a User Object.  If not then return None as
        required by Flask-Login.
        """
        """
        current_users = UserManager()
        user = current_users.get_user(userid)
        if user is not None:
            return User(user[1], user[2])
        return None
        """
        usr = User(userid, password, host=host, roles=roles, pl_password=pl_password)
        return usr

    def get_ip(self):
        return str(self.ip)

    def get_port(self):
        return str(self.port)

    def get_host(self):
        return str(self.host)

    def get_id(self):
        return str(self.id)

    def get_password(self):
        return str(self.password)

    def get_plain_password(self):
        return str(self.plain_password)

    def get_encrypted_password(self):

        from Crypto.Cipher import ARC4
        obj = ARC4.new(SECRET_KEY)
        return obj.encrypt(self.plain_password)

    @staticmethod
    def get_decrypted_password(cipher):

        from Crypto.Cipher import ARC4
        obj = ARC4.new(SECRET_KEY)
        return obj.decrypt(cipher)

    def authenticate(self, ip, port):
        """
        Instance method to check authenticity of .self user
        with citrusleaf
        """
        user = base.get_citrusleaf_cluster_user(ip, port,
                            user=self.get_id(), password=self.get_password())
        if user is None or user[0] != 0:
            return False
        else:
            return True

    def _fetch_roles(self, ip, port):

        self.roles = base.get_cluster_userroles(ip, port,
                            user=self.get_id(), password=self.get_password())

    def get_roles(self, ip, port):

        if self.roles: return self.roles
        self._fetch_roles(ip, port)
        return self.roles